/*
  This file was originally part of the GTA V SCRIPT HOOK SDK (c) Alexander Blade 2015, http://dev-c.com			

  Based on NativeSpeedo and inspired by SUPERHOT, this mod slows down time when the player is not moving.
  Feel free to improve, steal, or modify. 
 */

#include "script.h"

// gamespeed that is used when "stopped"
#define StopSpeed 0.001f
// treshold speed for player/car entity
#define StopTreshold 0.1f
// normal gamespeed
#define NormalSpeed 1.0f
// toggle mod key
#define ModHotkey VK_F5

bool modDisabled = true;

bool hotKeyPressed = false;

void update()
{
	// check for hotkey
	if (GetAsyncKeyState(ModHotkey) & 0x8000) {
		// only toggle when the key has been first pressed
		if (!hotKeyPressed) {
			modDisabled = !modDisabled;
			hotKeyPressed = true;
			// reset gamespeed on disable
			if (modDisabled)
				GAMEPLAY::SET_TIME_SCALE(NormalSpeed);
		}
	}
	else
		hotKeyPressed = false;

	if (modDisabled) {
		return;
	}

	Player player = PLAYER::PLAYER_ID();
	Ped playerPed = PLAYER::PLAYER_PED_ID();

	// check if player ped exists and control is on (e.g. not in a cutscene)
	if (!ENTITY::DOES_ENTITY_EXIST(playerPed) || !PLAYER::IS_PLAYER_CONTROL_ON(player)) {
		GAMEPLAY::SET_TIME_SCALE(NormalSpeed);
		return;
	}
	else

	// check for player ped death and player arrest
	if (ENTITY::IS_ENTITY_DEAD(playerPed) || PLAYER::IS_PLAYER_BEING_ARRESTED(player, TRUE)) {
		// this causes the death-scene to be played at normal speed!
		GAMEPLAY::SET_TIME_SCALE(NormalSpeed);
		return;
	}

	// check if the player is doing anything
	if (PED::IS_PED_GETTING_INTO_A_VEHICLE(playerPed) || PED::IS_PED_RELOADING(playerPed) || PED::IS_PED_RAGDOLL(playerPed) ||
		PED::IS_PED_VAULTING(playerPed) || PED::IS_PED_JUMPING_OUT_OF_VEHICLE(playerPed) || PED::IS_PED_JUMPING(playerPed) ||
		PED::IS_PED_PERFORMING_STEALTH_KILL(playerPed) || PED::IS_PED_SHOOTING(playerPed)) { 
		GAMEPLAY::SET_TIME_SCALE(NormalSpeed);
		return;
	}

	// check if any controls (wsad, fire, accelerate, brake) are pressed
	if (CONTROLS::IS_CONTROL_PRESSED(2, 8) || CONTROLS::IS_CONTROL_PRESSED(2, 9) || CONTROLS::IS_CONTROL_PRESSED(2, 32) || 
		CONTROLS::IS_CONTROL_PRESSED(2, 24) || CONTROLS::IS_CONTROL_PRESSED(2, 34) ) {
		GAMEPLAY::SET_TIME_SCALE(NormalSpeed);
		return;
	}

	float speed = 0.f;
	if (PED::IS_PED_IN_ANY_VEHICLE(playerPed, FALSE))
	{
		// check for acceleration / brakes
		if (CONTROLS::IS_CONTROL_PRESSED(2, 71) || CONTROLS::IS_CONTROL_PRESSED(2, 72)) {
			GAMEPLAY::SET_TIME_SCALE(NormalSpeed);
			return;
		}

		// when in a vehicle, we rather check the speed of the car (probably doesn't make a difference)
		Vehicle	car = PED::GET_VEHICLE_PED_IS_USING(playerPed);
		if (VEHICLE::_IS_VEHICLE_ENGINE_ON(car))	// this seems to return true when the player is still hotwiring the car
			speed = ENTITY::GET_ENTITY_SPEED(car);
	}
	else {
		speed = ENTITY::GET_ENTITY_SPEED(playerPed);
	}

	if (speed > StopTreshold)
		GAMEPLAY::SET_TIME_SCALE(NormalSpeed);
	else
		GAMEPLAY::SET_TIME_SCALE(StopSpeed);
}

void main()
{	
	while (true)
	{
		update();
		WAIT(0);
	}
}

void ScriptMain()
{
	main();
}
